function printStartScript( scriptName )
	DingoAlert( "Start " .. scriptName .. " script" ) ;
end

function printEndScript( scriptName )
	DingoAlert( "End " .. scriptName .. " script" ) ;
end


function tablegetn( tableName )
	-- lua 5.3.5 no longer has table.getn ... simulate here (#table might work too, but only on arrays?)
	if ( isLua53() ) then
		-- count up
		local counter = 0
		for index in pairs(tableName) do
			counter = counter + 1
		end
		return counter
	else
		return table.getn( tableName ) ;
	end
end

function mathatan2( atan2x, atan2y )
	-- lua 5.3.5 no longer has math.atan2 (now atan version can take one or two parameters)
	if ( isLua53() ) then
		return math.atan(atan2x, atan2y);
	else
		return math.atan2(atan2x, atan2y);
	end
end

-- Tasty Planet 4

function mars1_1_levelscript( )
	local scriptName = "mars1_1" ;
	printStartScript( scriptName ) ;

	local numQuarks = 5 ;

	local emitQuarkName = "quark" ;
	local area = 1.0 ; -- really small so always smaller than player
	addScriptedEntityEmitting( emitQuarkName, area ) ;

	numQuarksVarName = "numQuarks" ;
	numQuarksExist = 0 ;
	setScriptVariable( numQuarksVarName, numQuarksExist ) ;
	
	atomList = {} ;		
	local maxAtoms = 20 ;
	local numAtoms = 0 ;
	
	local removeEvery = 5 ;
	local timesSinceRemove = 0 ;
	
	while ( getCurrentLevelName( ) == scriptName ) do
		--DingoAlert( "mars1_1_levelscript loop" ) ;

		if ( numAtoms < maxAtoms ) then
			if ( randomDouble( 0.0, 1.0 ) < 0.2 ) then
				table.insert( atomList, createQuarkGroup( ) ) ;
				numAtoms = numAtoms + 1 ;
			end
		end

		local index = 1 ;
		local quarkIndex = 1 ;
		for index, atom in ipairs(atomList) do
			--DingoAlert( "change color: " .. index ) ;
			if ( randomDouble( 0.0, 1.0 ) < 0.3 ) then
				changeColor( atom ) ;
			end
		end

		if ( timesSinceRemove >= removeEvery ) then
			timesSinceRemove = 0 ;
			
			for index, atom in ipairs(atomList) do
				local anyAlive = false ;
				for quarkIndex, quarkName in ipairs(atom) do
					--DingoAlert( "quark : " .. quarkIndex .. " " .. quarkName ) ;

					if ( entityIsAlive( quarkName ) == true ) then
						anyAlive = true ;
					end
				end
				
				if ( anyAlive == false ) then
					--DingoAlert( "removing atom" ) ;					
					table.remove( atomList, index ) ;
					numAtoms = numAtoms - 1 ;
					break ;
				end
				
				index = index + 1 ;
			end
		else
			timesSinceRemove = timesSinceRemove + 1 ;
		end

	
		--local waitTime = randomDouble( 0.1, 0.6 ) ;
		wait( 0.1 ) ;
	end
	
	removeScriptedEntityEmitting( emitQuarkName ) ;
	
	printEndScript( scriptName ) ;
end

function createQuarkGroup( )
	local levelRight = 1400.0 ; 
	local levelLeft = -1400.0 ; 
	local levelBottom = 1400.0 ; 
	local levelTop = -1400.0 ; 
	local emitFromEdge = 150.0 ;
	local emitSize = 1000.0 ;

	local radius = 60.0 ;
	local degreesToRadians = 0.0174533 ;
	local angle = ( 360.0 / 3.0 ) * degreesToRadians ;
	local startAngle = randomDouble ( 0.0, 360.0 ) * degreesToRadians ;
	local posX1 = radius * math.cos( startAngle + angle ) ;
	local posY1 = radius * math.sin( startAngle + angle ) ;
	local posX2 = radius * math.cos( startAngle + angle*2 ) ;
	local posY2 = radius * math.sin( startAngle + angle*2 ) ;
	local posX3 = radius * math.cos( startAngle + angle*3 ) ;
	local posY3 = radius * math.sin( startAngle + angle*3 ) ;

	local centerX = 0.0 ;
	local centerY = 0.0 ;
	
	local side = math.floor(randomInt( 0, 4 )) ;
	if ( side == 0 ) then
		-- left
		centerX = levelLeft - emitFromEdge ;
		centerY = randomDouble( -emitSize, emitSize ) ;
	elseif ( side == 1 ) then
		-- right
		centerX = levelRight + emitFromEdge ;
		centerY = randomDouble( -emitSize, emitSize ) ;
	elseif ( side == 2 ) then
		-- top
		centerX = randomDouble( -emitSize, emitSize ) ;
		centerY = levelTop - emitFromEdge ;
	elseif ( side == 3 ) then
		-- bottom
		centerX = randomDouble( -emitSize, emitSize ) ;
		centerY = levelBottom + emitFromEdge ;
	end

	local angleToMiddle = mathatan2( -centerY, -centerX ) ;
	
	local angleRandomness = 30.0 * degreesToRadians ;
	local directionAngle = randomDouble( angleToMiddle - angleRandomness, angleToMiddle + angleRandomness ) ;
	local speed = randomDouble( 4.0, 8.0 ) ;
	
	local velX = speed * math.cos( directionAngle ) ;
	local velY = speed * math.sin( directionAngle ) ;
	local area = 4000 ;
	local angularVelocity = 0 ; --randomDouble( -300.0, 300.0 ) ;
	local priority = math.floor(randomInt( 100, 130 )) ;
	local scriptName1 = createEntity( "quark_base", centerX + posX1, centerY + posY1, 0.0, velX, velY, area, angularVelocity, priority, 1 ) ;
	local scriptName2 = createEntity( "quark_base", centerX + posX2, centerY + posY2, 0.0, velX, velY, area, angularVelocity, priority, 1 ) ;
	local scriptName3 = createEntity( "quark_base", centerX + posX3, centerY + posY3, 0.0, velX, velY, area, angularVelocity, priority, 1 ) ;	
	
	local listOfQuarks = { scriptName1, scriptName2, scriptName3 } ;
	
	changeColor( listOfQuarks ) ;
	
	return listOfQuarks ;
end

function changeColor( listOfQuarks )
	local redR = 1.0 ;
	local redG = 0.1 ;
	local redB = 0.1 ;
	local redA = 0.6 ;

	local greenR = 0.0 ;
	local greenG = 1.0 ;
	local greenB = 0.0 ;
	local greenA = 0.5 ;

	local blueR = 0.2 ;
	local blueG = 0.2 ;
	local blueB = 1.0 ;
	local blueA = 0.7 ;
	
	local effColScale = 0.7 ;

	local colourIndex = math.floor(randomInt( 0, 3 )) ;
	if ( colourIndex == 0 ) then
		addColourForColourizeIndex( listOfQuarks[1], 0, redR, redG, redB, redA ) ;
		setEffectColour( listOfQuarks[1], redR*effColScale, redG*effColScale, redB*effColScale ) ;
		addColourForColourizeIndex( listOfQuarks[2], 0, greenR, greenG, greenB, greenA ) ;
		setEffectColour( listOfQuarks[2], greenR*effColScale, greenG*effColScale, greenB*effColScale ) ;
		addColourForColourizeIndex( listOfQuarks[3], 0, blueR, blueG, blueB, blueA ) ;
		setEffectColour( listOfQuarks[3], blueR*effColScale, blueG*effColScale, blueB*effColScale ) ;
	elseif ( colourIndex == 1 ) then
		addColourForColourizeIndex( listOfQuarks[2], 0, redR, redG, redB, redA ) ;
		setEffectColour( listOfQuarks[2], redR*effColScale, redG*effColScale, redB*effColScale ) ;
		addColourForColourizeIndex( listOfQuarks[3], 0, greenR, greenG, greenB, greenA ) ;
		setEffectColour( listOfQuarks[3], greenR*effColScale, greenG*effColScale, greenB*effColScale ) ;
		addColourForColourizeIndex( listOfQuarks[1], 0, blueR, blueG, blueB, blueA ) ;
		setEffectColour( listOfQuarks[1], blueR*effColScale, blueG*effColScale, blueB*effColScale ) ;
	elseif ( colourIndex == 2 ) then
		addColourForColourizeIndex( listOfQuarks[3], 0, redR, redG, redB, redA ) ;
		setEffectColour( listOfQuarks[3], redR*effColScale, redG*effColScale, redB*effColScale ) ;
		addColourForColourizeIndex( listOfQuarks[1], 0, greenR, greenG, greenB, greenA ) ;
		setEffectColour( listOfQuarks[1], greenR*effColScale, greenG*effColScale, greenB*effColScale ) ;
		addColourForColourizeIndex( listOfQuarks[2], 0, blueR, blueG, blueB, blueA ) ;
		setEffectColour( listOfQuarks[2], blueR*effColScale, blueG*effColScale, blueB*effColScale ) ;
	end
end


function mars1_51_levelscript( )
	local scriptName = "mars1_51" ;
	printStartScript( scriptName ) ;

	local numUniverses = 5 ;

	local emitUniverseName = "universe" ;
	local area = 1.0 ; -- really small so always smaller than player
	addScriptedEntityEmitting( emitUniverseName, area ) ;

	numUniversesVarName = "numUniverses" ;
	numUniversesExist = 0 ;
	setScriptVariable( numUniversesVarName, numUniversesExist ) ;
	
	utomList = {} ;		
	local maxUtoms = 20 ;
	local numUtoms = 0 ;
	
	local removeEvery = 5 ;
	local timesSinceRemove = 0 ;

	-- create initial universe that we just came from
	table.insert( utomList, createUniverseGroup( true ) ) ;
	numUtoms = numUtoms + 1 ;
	
	while ( getCurrentLevelName( ) == scriptName ) do
		--DingoAlert( "mars1_1_levelscript loop" ) ;

		if ( numUtoms < maxUtoms ) then
			if ( randomDouble( 0.0, 1.0 ) < 0.2 ) then
				table.insert( utomList, createUniverseGroup( false ) ) ;
				numUtoms = numUtoms + 1 ;
			end
		end

		local index = 1 ;
		local universeIndex = 1 ;
		for index, utom in ipairs(utomList) do
			--DingoAlert( "change color: " .. index ) ;
			if ( randomDouble( 0.0, 1.0 ) < 0.3 ) then
				changeUColor( utom ) ;
			end
		end

		if ( timesSinceRemove >= removeEvery ) then
			timesSinceRemove = 0 ;
			
			for index, utom in ipairs(utomList) do
				local anyAlive = false ;
				for universeIndex, universeName in ipairs(utom) do
					--DingoAlert( "universe : " .. universeIndex .. " " .. universeName ) ;

					if ( entityIsAlive( universeName ) == true ) then
						anyAlive = true ;
					end
				end
				
				if ( anyAlive == false ) then
					--DingoAlert( "removing utom" ) ;					
					table.remove( utomList, index ) ;
					numUtoms = numUtoms - 1 ;
					break ;
				end
				
				index = index + 1 ;
			end
		else
			timesSinceRemove = timesSinceRemove + 1 ;
		end

	
		--local waitTime = randomDouble( 0.1, 0.6 ) ;
		wait( 0.1 ) ;
	end
	
	removeScriptedEntityEmitting( emitUniverseName ) ;
	
	printEndScript( scriptName ) ;
end

function createUniverseGroup( initialUniverse )
	local levelRight = 1400.0 ; 
	local levelLeft = -1400.0 ; 
	local levelBottom = 1400.0 ; 
	local levelTop = -1400.0 ; 
	local emitFromEdge = 150.0 ;
	local emitSize = 1000.0 ;

	local radius = 60.0 ;
	local degreesToRadians = 0.0174533 ;
	local angle = ( 360.0 / 3.0 ) * degreesToRadians ;
	local startAngle = randomDouble ( 0.0, 360.0 ) * degreesToRadians ;
	
	if ( initialUniverse ) then
		startAngle = 0.0 ;
	end
	
	local posX1 = radius * math.cos( startAngle + angle ) ;
	local posY1 = radius * math.sin( startAngle + angle ) ;
	local posX2 = radius * math.cos( startAngle + angle*2 ) ;
	local posY2 = radius * math.sin( startAngle + angle*2 ) ;
	local posX3 = radius * math.cos( startAngle + angle*3 ) ;
	local posY3 = radius * math.sin( startAngle + angle*3 ) ;

	local centerX = 0.0 ;
	local centerY = 0.0 ;
	
	local side = math.floor(randomInt( 0, 4 )) ;
	if ( side == 0 ) then
		-- left
		centerX = levelLeft - emitFromEdge ;
		centerY = randomDouble( -emitSize, emitSize ) ;
	elseif ( side == 1 ) then
		-- right
		centerX = levelRight + emitFromEdge ;
		centerY = randomDouble( -emitSize, emitSize ) ;
	elseif ( side == 2 ) then
		-- top
		centerX = randomDouble( -emitSize, emitSize ) ;
		centerY = levelTop - emitFromEdge ;
	elseif ( side == 3 ) then
		-- bottom
		centerX = randomDouble( -emitSize, emitSize ) ;
		centerY = levelBottom + emitFromEdge ;
	end
	

	local angleToMiddle = mathatan2( -centerY, -centerX ) ;
	
	local angleRandomness = 30.0 * degreesToRadians ;
	local directionAngle = randomDouble( angleToMiddle - angleRandomness, angleToMiddle + angleRandomness ) ;
	local speed = randomDouble( 4.0, 8.0 ) ;

	if ( initialUniverse ) then
		speed = 0.1 ;
	end
	
	local velX = speed * math.cos( directionAngle ) ;
	local velY = speed * math.sin( directionAngle ) ;
	local customArea = 0.05 ;
	local normalArea = 0.264252 ;
	local area = 4000 * ( customArea / normalArea ) ;
	local angularVelocity = 0 ; --randomDouble( -300.0, 300.0 ) ;
	
	if ( initialUniverse ) then
		centerX = 140.0 ; -- 130.0 -- 115.0
		centerY = 0.0 ;
	end

	local scriptName1 = "nothingcreated" ;
	local scriptName2 = "nothingcreated" ;
	local scriptName3 = "nothingcreated" ;	
	
	local priority = math.floor(randomInt( 100, 130 )) ;
	
	scriptName1 = createEntity( "universe_base", centerX + posX1, centerY + posY1, 0.0, velX, velY, area, angularVelocity, priority, 1 ) ;
	scriptName2 = createEntity( "universe_base", centerX + posX2, centerY + posY2, 0.0, velX, velY, area, angularVelocity, priority, 1 ) ;

	if ( not initialUniverse ) then
		-- dont create this one for initial universe group (we came from inside it and ate it already)
		scriptName3 = createEntity( "universe_base", centerX + posX3, centerY + posY3, 0.0, velX, velY, area, angularVelocity, priority, 1 ) ;	
	end
	
	local listOfUniverses = { scriptName1, scriptName2, scriptName3 } ;
	
	changeUColor( listOfUniverses ) ;
	
	return listOfUniverses ;
end

function changeUColor( listOfUniverses )
	local redR = 1.0 ;
	local redG = 0.1 ;
	local redB = 0.1 ;
	local redA = 0.6 ;

	local greenR = 0.0 ;
	local greenG = 1.0 ;
	local greenB = 0.0 ;
	local greenA = 0.5 ;

	local blueR = 0.2 ;
	local blueG = 0.2 ;
	local blueB = 1.0 ;
	local blueA = 0.7 ;
	
	local effColScale = 0.7 ;

	local colourIndex = math.floor(randomInt( 0, 3 )) ;
	if ( colourIndex == 0 ) then
		addColourForColourizeIndex( listOfUniverses[1], 0, redR, redG, redB, redA ) ;
		setEffectColour( listOfUniverses[1], redR*effColScale, redG*effColScale, redB*effColScale ) ;
		addColourForColourizeIndex( listOfUniverses[2], 0, greenR, greenG, greenB, greenA ) ;
		setEffectColour( listOfUniverses[2], greenR*effColScale, greenG*effColScale, greenB*effColScale ) ;
		addColourForColourizeIndex( listOfUniverses[3], 0, blueR, blueG, blueB, blueA ) ;
		setEffectColour( listOfUniverses[3], blueR*effColScale, blueG*effColScale, blueB*effColScale ) ;
	elseif ( colourIndex == 1 ) then
		addColourForColourizeIndex( listOfUniverses[2], 0, redR, redG, redB, redA ) ;
		setEffectColour( listOfUniverses[2], redR*effColScale, redG*effColScale, redB*effColScale ) ;
		addColourForColourizeIndex( listOfUniverses[3], 0, greenR, greenG, greenB, greenA ) ;
		setEffectColour( listOfUniverses[3], greenR*effColScale, greenG*effColScale, greenB*effColScale ) ;
		addColourForColourizeIndex( listOfUniverses[1], 0, blueR, blueG, blueB, blueA ) ;
		setEffectColour( listOfUniverses[1], blueR*effColScale, blueG*effColScale, blueB*effColScale ) ;
	elseif ( colourIndex == 2 ) then
		addColourForColourizeIndex( listOfUniverses[3], 0, redR, redG, redB, redA ) ;
		setEffectColour( listOfUniverses[3], redR*effColScale, redG*effColScale, redB*effColScale ) ;
		addColourForColourizeIndex( listOfUniverses[1], 0, greenR, greenG, greenB, greenA ) ;
		setEffectColour( listOfUniverses[1], greenR*effColScale, greenG*effColScale, greenB*effColScale ) ;
		addColourForColourizeIndex( listOfUniverses[2], 0, blueR, blueG, blueB, blueA ) ;
		setEffectColour( listOfUniverses[2], blueR*effColScale, blueG*effColScale, blueB*effColScale ) ;
	end
end

function soccer1_1_levelscript( )
	DingoAlert( "Running soccer1_1_levelscript Script" ) ;

	local scriptName = "soccer1_1" ;
	printStartScript( scriptName ) ;

	trackLastEntityPositions( ) ;

	local emitName = "earth" ;
	local area = 1600.0 ; -- really small so always smaller than player
	addScriptedEntityEmitting( emitName, area ) ;

	local minTimeShot = 0.3 ; -- 0.0
	local maxTimeShot = 1.0 ; -- 0.7
	
	local boostSlowDown = 1.0 ;
	
	while ( true ) do
		local timeToNextShot = randomDouble( minTimeShot, maxTimeShot ) ;
		wait( timeToNextShot ) ;
		
		local objectName = "earth_soccer1_1" ;
		local posX = -1030.0 ;
		local posY = 0.0 ; --randomDouble( -230.0, 230.0 ) ;
		local speed = randomDouble( 35.0, 60.0 ) ;
		
		if ( isBoosterSlowEnemiesActive( ) ) then
			boostSlowDown = 0.5 ;
		end

		speed = speed * boostSlowDown ;

		local launchAngleMin = -10.0 ;
		local luanchAngleMax = 10.0 ;
		
		local rValue = math.floor(randomInt( 0, 5 )) ;
		if ( rValue == 0 ) then
			posY = -230.0 ;
			launchAngleMin = 0.0 ;
			launchAngleMax = 5.0 ;
		elseif ( rValue == 1 ) then
			posY = -120.0 ;
			launchAngleMin = -1.0 ;
			launchAngleMax = 5.0 ;
		elseif ( rValue == 2 ) then
			posY = 0.0 ;
			launchAngleMin = -3.0 ;
			launchAngleMax = 3.0 ;
		elseif ( rValue == 3 ) then
			posY = 120.0 ;
			launchAngleMin = -5.0 ;
			launchAngleMax = 1.0 ;
		elseif ( rValue == 4 ) then
			posY = 230.0 ;
			launchAngleMin = -5.0 ;
			launchAngleMax = 0.0 ;
		end
		
		local launchAngleDegrees = randomDouble( launchAngleMin, launchAngleMax ) ;
		
		local launchAngleRadians = launchAngleDegrees * ( 3.14159/180.0 ) ;
		local velX = speed * math.cos( launchAngleRadians ) ;
		local velY = - speed * math.sin( launchAngleRadians ) ;
		
		local angularVelocity = randomDouble( -300.0, 300.0 ) ;
		local priority = math.floor(randomInt( 100, 130 )) ;
		local startAngle = randomDouble( -100.0, 100.0 ) ;
		local ballName = createEntity( objectName, posX, posY, startAngle, velX, velY, area, angularVelocity, priority ) ;
		setEntityPoints( ballName, true, 100 ) ;
		
		local timeToCheckForScore = 2.5 ;
		timeToCheckForScore = timeToCheckForScore / boostSlowDown ;
		local scored = false ;
		local ballDone = false ;
		while ( not ballDone and timeToCheckForScore > 0.0 ) do
			timeToCheckForScore = timeToCheckForScore - getDeltaT( ) ;
			
			if ( scored == false ) then
				if ( entityIsAlive( ballName ) ) then
					local entityPosX, entityPosY = getEntityPos( ballName ) ;
					if ( entityPosX > 1070.0 ) then
						scored = true ;
						ballDone = true ;
						removePoints( 30, 950.0, entityPosY ) ;
						playSound( "negativechime" ) ;
					end
				else
					-- ball is not alive - call it done
					ballDone = true ;
				end
			end
	
			coroutine.yield( ) ;
		end
	end
	
	removeScriptedEntityEmitting( emitName ) ;
	
	printEndScript( scriptName ) ;
end

function goalieScript( )
	local goalieSpeed = 400.0 ;
	
	if ( isBoosterSlowEnemiesActive( ) ) then
		goalieSpeed = goalieSpeed / 8.0 ;
	end
	
	local goalieName = "pasta1" ;

	local goalieDefaultX = 900.0 ;
	local goalieDefaultY = 0.0 ;

	while ( true ) do
		if ( entityIsAlive( goalieName ) ) then
			local goaliePosX, goaliePosY = getEntityPos( goalieName ) ;

			local destX = goalieDefaultX ;
			local destY = goalieDefaultY ;

			if ( entityIsAlive( soccer2BallName ) ) then
				local ballPosX, ballPosY = getEntityPos( soccer2BallName ) ;
				if ( ballPosX > 350.0 and ballPosX < 1000.0 and ballPosY > -560.0 and ballPosY < 530.0 ) then
					-- ball inside tracking zone
					destX = ballPosX ;
					destY = ballPosY ;
				end
			end

			local dirX = destX - goaliePosX ;
			local dirY = destY - goaliePosY ;
			
			local dist = math.sqrt( dirX * dirX + dirY * dirY ) ;
			if ( dist < 10.0 ) then
				setEntityVel( goalieName, 0.0, 0.0 ) ;
			else
				local normalizedX = dirX / dist ;
				local normalizedY = dirY / dist ;
				
				local velX = normalizedX * goalieSpeed ;
				local velY = normalizedY * goalieSpeed ;
				
				setEntityVel( goalieName, velX, velY ) ;
			end
		end
		
		coroutine.yield( ) ;
	end
end

function goalieTrackWin( )
	local emitted = false ;
	while ( not emitted ) do
		if ( getScore( ) >= 5000 ) then
			
			while ( true ) do
				local objectName = "mars_soccer2_1" ;
				local soccerBallArea = 1000.0 ;	

				local ballLifeTime = 0.0 ;
				
				local posX = -180.0 ;
				local posY = -870.0 ;

				local speed = 7.0 ;	
				local launchAngleDegrees = randomDouble( 240.0, 300.0 ) ;
				local launchAngleRadians = launchAngleDegrees * ( 3.14159/180.0 ) ;
				local velX = speed * math.cos( launchAngleRadians ) ;
				local velY = - speed * math.sin( launchAngleRadians ) ;	
				
				local angularVelocity = randomDouble( -300.0, 300.0 ) ;
				local priority = math.floor(randomInt( 100, 130 )) ;
				local startAngle = randomDouble( -100.0, 100.0 ) ;
				local tempName = createEntity( objectName, posX, posY, startAngle, velX, velY, soccerBallArea, angularVelocity, priority ) ;
				setEntityPoints( tempName, true, 1 ) ;
				
				wait( 0.5 ) ;
			end
		
			emitted	= true ;
		end

		wait ( 0.5 ) ;
	end
end

function soccer2_1_levelscript( )
	local scriptName = "soccer2_1" ;
	printStartScript( scriptName ) ;

	local emitName = "earth" ;
	addScriptedEntityEmitting( emitName, 1.0 ) ;

	trackLastEntityPositions( ) ;

	local minTimeShot = 0.0 ;
	local maxTimeShot = 0.7 ;

	soccer2BallName = "" ;

	activateScriptMessage( "goalieScript" ) ;
	activateScriptMessage( "goalieTrackWin" ) ;

	while ( true ) do
		local objectName = "earth_soccer2_1" ;
		local soccerBallArea = 2600.0 ;	

		local ballLifeTime = 0.0 ;
		
		local posX = -180.0 ;
		local posY = -870.0 ;

		local speed = 2.0 ;	
		local launchAngleDegrees = 270.0 ;
		local launchAngleRadians = launchAngleDegrees * ( 3.14159/180.0 ) ;
		local velX = speed * math.cos( launchAngleRadians ) ;
		local velY = - speed * math.sin( launchAngleRadians ) ;	
		
		local angularVelocity = randomDouble( -300.0, 300.0 ) ;
		local priority = math.floor(randomInt( 100, 130 )) ;
		local startAngle = randomDouble( -100.0, 100.0 ) ;
		soccer2BallName = createEntity( objectName, posX, posY, startAngle, velX, velY, soccerBallArea, angularVelocity, priority ) ;
		setEntityPoints( soccer2BallName, true, 1 ) ;
		
		local ballDone = false ;
		while ( entityIsAlive( soccer2BallName ) and ( not ballDone ) ) do
			local ballPosX, ballPosY = getEntityPos( soccer2BallName ) ;
			
			if ( ballPosY > -510.0 and ballPosY < 460.0 and ballPosX > 1000.0 ) then
				-- scored
				if ( getScore( ) < 5000 ) then -- don't let player keep scoring
					addPoints( 1000, 900.0, ballPosY ) ;
					playSound( "positivechime" ) ;
				end
				ballDone = true ;
			elseif ( ballPosX < -990.0 or ballPosX > 1005.0 or ballPosY < -790.0 or ballPosY > 790.0 ) then
				-- ball almost outside level
				local checkOutOfBoundsAfter = 2.0 ;
				if ( getEntityLivedFor( soccer2BallName ) > checkOutOfBoundsAfter ) then
					destX = ballPosX ;
					destY = ballPosY ;
					ballDone = true ;
				end
			end

			coroutine.yield( ) ;
		end
		
		wait( 0.5 ) ;
	end
	
	removeScriptedEntityEmitting( emitName ) ;
	
	printEndScript( scriptName ) ;
end

function spaceship2_1_levelscript( )
	local scriptName = "spaceship2_1" ;
	printStartScript( scriptName ) ;

	local startingLevel = "spaceship2_1" ;

	local emitName = "smallstuff" ;
	addScriptedEntityEmitting( emitName, 1.0 ) ;

	local emitX = -1024.0 ;
	local emitY = -80.0 ;
	local emitAngleDeg = 90.0 ;
	local emitRangeDeg = 23.0 ;
	
	unopenedAirlocks = { 'door1', 'door2', 'door3', 'door4', 'door5', 'door6', 'door7' } ;	
	
	while ( getCurrentLevelName( ) == startingLevel and tablegetn( unopenedAirlocks ) > 0 ) do
		for index, doorName in ipairs(unopenedAirlocks) do
			if ( not entityIsAlive( doorName ) ) then
			
				if ( doorName == 'door1' ) then
					emitX = -1024.0 ;
					emitY = -80.0 ;
					emitAngleDeg = 90.0 ;
				elseif ( doorName == 'door2' ) then
					emitX = -1030.0 ;
					emitY = 415.0 ;
					emitAngleDeg = 270.0 ;
				elseif ( doorName == 'door3' ) then
					emitX = -475.0 ;
					emitY = 430.0 ;
					emitAngleDeg = 270.0 ;
				elseif ( doorName == 'door4' ) then
					emitX = 78.0 ;
					emitY = -87.0 ;
					emitAngleDeg = 90.0 ;
				elseif ( doorName == 'door5' ) then
					emitX = 70.0 ;
					emitY = 414.0 ;
					emitAngleDeg = 270.0 ;
				elseif ( doorName == 'door6' ) then
					emitX = 609.0 ;
					emitY = 90.0 ;
					emitAngleDeg = 90.0 ;
				elseif ( doorName == 'door7' ) then
					emitX = 1235.0 ;
					emitY = 233.0 ;
					emitAngleDeg = 270.0 ;
				end
			
				local numToEmit = 15 ;
				local emitted = 0 ;
				while ( emitted < numToEmit and getCurrentLevelName( ) == startingLevel ) do
					local objectName = "" ;
					
					local objectIndex = math.floor(randomInt( 0, 2 )) ;
					if ( objectIndex == 0 ) then
						objectName = "gastank_spaceship2" ;
					elseif ( objectIndex == 1 ) then
						objectName = "storagecontainer_spaceship2" ;
					end
					
					local objectArea = 200.0 ;	

					local emitWidth = 100.0 ;
					local posX = randomDouble( emitX - emitWidth*0.5, emitX + emitWidth*0.5 ) ;
					local posY = emitY ;

					local speed = 20.0 ;	
					local launchAngleDegrees = randomDouble( emitAngleDeg - emitRangeDeg, emitAngleDeg + emitRangeDeg ) ;
					local launchAngleRadians = launchAngleDegrees * ( 3.14159/180.0 ) ;
					local velX = speed * math.cos( launchAngleRadians ) ;
					local velY = - speed * math.sin( launchAngleRadians ) ;	
					
					local angularVelocity = randomDouble( -300.0, 300.0 ) ;
					local priority = math.floor(randomInt( 100, 130 )) ;
					local startAngle = randomDouble( -100.0, 100.0 ) ;
					local transferLevels = 0 ;
					local tempName = createEntity( objectName, posX, posY, startAngle, velX, velY, objectArea, angularVelocity, priority, transferLevels ) ;

					setEntityPoints( tempName, true, 10 ) ;

					emitted = emitted + 1 ;
					wait( 0.05 ) ;
				end	
				
				table.remove( unopenedAirlocks, index ) ;
				
				break ;
			end
		end
		
		coroutine.yield( ) ;
	end
	
	removeScriptedEntityEmitting( emitName ) ;
	
	printEndScript( scriptName ) ;
end

function throwAppleOrPinecone( throwerName, throwPosX, throwPosY, launchAngleDegrees )
	local attackAnimLength = 1.0 ;
	local attackAnimBlend = 0.2 ;
	local windupTime = 0.45 ;
	local velMult = 4.0 ;

	local objectName = "pinecone_throwing1_2" ;
	local area = 500.0 ;
	
	if ( math.floor(randomInt( 0, 2 )) == 1 ) then
		objectName = "apple_throwing1_2" ;
		area = 750.0 ;
		velMult = 2.5 ;
	end

	playAnimBlend( throwerName, "attack", attackAnimLength, false, attackAnimBlend ) ;
	wait( attackAnimBlend + windupTime ) ;
	
	local posX = throwPosX ;
	local posY = throwPosY ;
	local speed = randomDouble( 10.0, 17.0 ) * velMult ;
		
	local launchAngleRadians = launchAngleDegrees * ( 3.14159/180.0 ) ;
	local velX = speed * math.cos( launchAngleRadians ) ;
	local velY = - speed * math.sin( launchAngleRadians ) ;	
	
	local angularVelocity = randomDouble( -300.0, 300.0 ) ;
	local priority = math.floor(randomInt( 100, 130 )) ;
	local startAngle = randomDouble( -100.0, 100.0 ) ;
	createEntity( objectName, posX, posY, startAngle, velX, velY, area, angularVelocity, priority ) ;

	wait( attackAnimLength - windupTime ) ;
	playAnimBlend( throwerName, "idle", 1.0, true, 0.5 ) ;
end

function throwerScript( throwerName, throwPosX, throwPosY, throwAngle )
	playAnim( throwerName, "idle", 1.0, true ) ;

	local throwingObject = throwerName .. "appleorpine" ;
	local area = 1.0 ; -- really small so always smaller than player
	addScriptedEntityEmitting( throwingObject, area ) ;

	local minThrowTime = 2.0 ;
	local maxThrowTime = 5.0 ;	
	local timeToNextThrow = randomDouble( minThrowTime, maxThrowTime )
	while ( true ) do
		wait( timeToNextThrow ) ;
		
		if ( entityIsAlive( throwerName ) ) then
			throwAppleOrPinecone( throwerName, throwPosX, throwPosY, throwAngle ) ;
		end
		
		timeToNextThrow = randomDouble( minThrowTime, maxThrowTime )
		
		coroutine.yield( ) ;
	end

	removeScriptedEntityEmitting( throwingObject ) ;
end

function throwing1_2_levelscript( )
	DingoAlert( "Running throwing1_1 Script" ) ;

	local startingLevel = getCurrentLevelName( ) ;

	activateScriptMessage( "throwerScript", "thrower1", -1300.0, 1440.0, 45.0 ) ;
	activateScriptMessage( "throwerScript", "thrower2", 220.0, 1452.0, 135.0 ) ;
end

function balloon1_2_levelscript( )
	DingoAlert( "Running balloon1_2 Script" ) ;

	playAnim( "keeper1", "custom1", 1.0, true ) ;
	playAnim( "keeper2", "custom1", 1.07, true ) ;
end


function stacks1_1_levelscript( )
	DingoAlert( "Running stacks1_1 Script" ) ;

	local startingLevel = getCurrentLevelName( ) ;

	undisturbedHiveList = { 'hive1', 'hive2', 'hive3', 'hive4', 'hive5', 'hive6', 'hive7', 'hive8', 'hive9', 'hive10', 'hive11', 'hive12', 'hive13', 'hive14', 'hive15', 'hive16', 'hive17', 'hive18' } ;	

	local timeBetweenChecks = 0.1 ;
	local closeToGroundY = 2310.0 ;
	local hasFallenY = 1900.0 ;
	
	local beeObject = "emittingbee" ;
	local area = 300.0 ;
	addScriptedEntityEmitting( beeObject, area ) ;
	
	local index = 1 ;
	while ( tablegetn(undisturbedHiveList) > 0 ) do
	
		local listToRemove = { } ;
	
		for index, hiveName in ipairs(undisturbedHiveList) do
			local entityPosX, entityPosY = getEntityPos( hiveName ) ;
			local entityVelX, entityVelY = getEntityVel( hiveName ) ;
			local atGround = entityPosY > closeToGroundY ;
			local fallenAndStopped = entityPosY > hasFallenY and math.abs( entityVelY ) < 20.0 ; -- backup if they stack on top of each other
			if ( atGround or fallenAndStopped ) then
				local velX = 0.0 ;
				local velY = 0.0 ;
				local priority = 0 ; -- below
				createEntity( "beenpc_stacks1_1", entityPosX, entityPosY, 0.0, velX, velY, area, 0.0, priority ) ;
				createEntity( "beenpc_stacks1_1", entityPosX+20.0, entityPosY, 0.0, velX, velY, area, 0.0, priority ) ;
				createEntity( "beenpc_stacks1_1", entityPosX-20.0, entityPosY, 0.0, velX, velY, area, 0.0, priority ) ;
				createEntity( "beenpc_stacks1_1", entityPosX+40.0, entityPosY, 0.0, velX, velY, area, 0.0, priority ) ;
				createEntity( "beenpc_stacks1_1", entityPosX-40.0, entityPosY, 0.0, velX, velY, area, 0.0, priority ) ;
				
				table.insert( listToRemove, index ) ;
			end
		end
		
		for index, indexInHiveTable in ipairs(listToRemove) do
			DingoAlert( "removing" .. indexInHiveTable ) ;
			table.remove( undisturbedHiveList, indexInHiveTable ) ;
		end
		
		wait( timeBetweenChecks ) ;
	end
	
	-- no more hives left
	removeScriptedEntityEmitting( beeObject ) ;

end

function stacks3_1_levelscript( )
	DingoAlert( "Running stacks3_1 Script" ) ;

	local startingLevel = getCurrentLevelName( ) ;

	local hiveName = 'hive1' ;
	
	local beeObject = "emittingbee" ;
	local area = 300.0 ;
	addScriptedEntityEmitting( beeObject, area ) ;

	local timeBetweenEmits = 1.0 ;
	local timeSinceEmit = 1000.0 ;
	local emitAtSpeed = 200.0 ;

	local numberOfEmits = 0 ;

	while ( entityIsAlive( hiveName ) ) do
		timeSinceEmit = timeSinceEmit + getDeltaT( ) ;
		if ( timeSinceEmit > timeBetweenEmits ) then
			-- time has passed, try see if hive is moving and should emit

			local entityVelX, entityVelY = getEntityVel( hiveName ) ;
			
			local speed = math.sqrt( entityVelX * entityVelX + entityVelY * entityVelY ) ;
			if ( speed > emitAtSpeed ) then
				local entityPosX, entityPosY = getEntityPos( hiveName ) ;
				local velX = 0.0 ;
				local velY = -5.0 ;
				local priority = 0 ; -- below
				createEntity( "beenpc_stacks3_1", entityPosX, entityPosY-20, 0.0, velX, velY, area, 0.0, priority ) ;
				createEntity( "beenpc_stacks3_1", entityPosX+20.0, entityPosY-20, 0.0, velX, velY, area, 0.0, priority ) ;
				createEntity( "beenpc_stacks3_1", entityPosX-20.0, entityPosY-20, 0.0, velX, velY, area, 0.0, priority ) ;
				createEntity( "beenpc_stacks3_1", entityPosX+40.0, entityPosY-20, 0.0, velX, velY, area, 0.0, priority ) ;
				createEntity( "beenpc_stacks3_1", entityPosX-40.0, entityPosY-20, 0.0, velX, velY, area, 0.0, priority ) ;
				
				numberOfEmits = numberOfEmits + 1 ;
				
				if ( numberOfEmits == 10 ) then
					createEntity( "beenpc_stacks3_big", entityPosX, entityPosY-40, 90.0, 0.0, -10.0, 5000.0, 0.0, priority ) ;
				end
				
				timeSinceEmit = 0.0 ;
			end
		end
		
		coroutine.yield( ) ;
	end
	
	removeScriptedEntityEmitting( beeObject ) ;
end




-- caribbean scripts
function containers1_2_levelscript( )
	DingoAlert( "Running containers1_2 Script" ) ;

	local startingLevel = getCurrentLevelName( ) ;

	ships = { 'containership1', 'containership2', 'containership3' } ;	
	
	local emitObject = "steelshippingcontainer_containers1_2" ;
	local area = 62.0 ;
	addScriptedEntityEmitting( emitObject, area ) ;

	local minEmitTime = 0.5 ;
	local maxEmitTime = 1.5 ;
	local timeToNextEmit = randomDouble( minEmitTime, maxEmitTime ) ;

	while ( true ) do
		timeToNextEmit = timeToNextEmit - getDeltaT( ) ;
		
		if ( timeToNextEmit < 0.0 ) then
			local emitIndex = math.floor(randomInt( 1, tablegetn(ships) + 1 )) ;			
			local shipName = ships[ emitIndex ] ;

			if ( entityIsAlive( shipName ) ) then
				local entityPosX, entityPosY = getEntityPos( shipName ) ;
				local velX = 0.0 ;
				local velY = 0.0 ;
				local angularVel = randomDouble( -40.0, 40.0 ) ;
				local priority = 0 ; -- below
				createEntity( emitObject, entityPosX, entityPosY, 0.0, velX, velY, area, angularVel, priority ) ;
			end

			timeToNextEmit = randomDouble( minEmitTime, maxEmitTime ) ;
		end
		
		coroutine.yield( ) ;		
	end
	
	removeScriptedEntityEmitting( emitObject ) ;
end

function garbageship1_2_levelscript( )
	DingoAlert( "Running garbageship1_2 Script" ) ;

	local startingLevel = getCurrentLevelName( ) ;

	ships = { 'garbageship1', 'garbageship2', 'garbageship3' } ;	
	
	local emitObject = "carhusk_garbageship1_2" ;
	local area = 117.0 ;
	addScriptedEntityEmitting( emitObject, area ) ;

	local minEmitTime = 0.5 ;
	local maxEmitTime = 1.5 ;
	local timeToNextEmit = randomDouble( minEmitTime, maxEmitTime ) ;

	while ( true ) do
		timeToNextEmit = timeToNextEmit - getDeltaT( ) ;
		
		if ( timeToNextEmit < 0.0 ) then
			local emitIndex = math.floor(randomInt( 1, tablegetn(ships) + 1 )) ;			
			local shipName = ships[ emitIndex ] ;

			if ( entityIsAlive( shipName ) ) then
				local entityPosX, entityPosY = getEntityPos( shipName ) ;
				local velX = 0.0 ;
				local velY = 0.0 ;
				local angularVel = randomDouble( -40.0, 40.0 ) ;
				local priority = 0 ; -- below
				createEntity( emitObject, entityPosX, entityPosY, 0.0, velX, velY, area, angularVel, priority ) ;
			end

			timeToNextEmit = randomDouble( minEmitTime, maxEmitTime ) ;
		end
		
		coroutine.yield( ) ;		
	end
	
	removeScriptedEntityEmitting( emitObject ) ;
end

function dynamite1_2_levelscript( )
	DingoAlert( "Running dynamite1_2 Script" ) ;

	local startingLevel = getCurrentLevelName( ) ;

	ships = { 'heli1', 'heli2', 'heli3' } ;	
	
	local emitObject = "dynamite_dynamite1_1" ;
	local area = 160.0 ; --128.0 ;
	addScriptedEntityEmitting( emitObject, area ) ;

	local minEmitTime = 0.5 ;
	local maxEmitTime = 1.0 ;
	local timeToNextEmit = randomDouble( minEmitTime, maxEmitTime ) ;
					
	while ( true ) do
		timeToNextEmit = timeToNextEmit - getDeltaT( ) ;
		
		if ( timeToNextEmit < 0.0 ) then
			local emitIndex = math.floor(randomInt( 1, tablegetn(ships) + 1 )) ;			
			local shipName = ships[ emitIndex ] ;

			if ( entityIsAlive( shipName ) ) then
				--local entityPosX, entityPosY = getEntityPos( shipName ) ;
				local entityPosX, entityPosY = getShotPos( shipName, 0.3, 0.05 ) ;
				local velX = 0.0 ;
				local velY = 0.0 ;
				local angularVel = randomDouble( -40.0, 40.0 ) ;
				local priority = 0 ; -- below
				createEntity( emitObject, entityPosX, entityPosY, 0.0, velX, velY, area, angularVel, priority ) ;
			end

			timeToNextEmit = randomDouble( minEmitTime, maxEmitTime ) ;
		end
		
		coroutine.yield( ) ;		
	end
	
	removeScriptedEntityEmitting( emitObject ) ;
end

function removeDeadFromList( entityList )
	local foundAlive = true ;
	while ( foundAlive ) do
		foundAlive = false ;
		for index, entityName in ipairs(entityList) do
			if ( entityIsAlive( entityName ) == false ) then
				table.remove( entityList, index ) ;
				foundAlive = true ;
				break ;
			end
		end
	end
end

function sharkdive1_1_levelscript( )
	DingoAlert( "Running sharkdive1_1 Script" ) ;

	local startingLevel = getCurrentLevelName( ) ;

	ships = { 'diveboat1', 'diveboat2', 'diveboat3' } ;	
	
	local emitObject = "scuba_sharkdive1_1" ;
	local area = 1677.0 ; --128.0 ;
	addScriptedEntityEmitting( emitObject, area ) ;

	local minEmitTime = 0.5 ;
	local maxEmitTime = 1.0 ;
	local timeToNextEmit = randomDouble( minEmitTime, maxEmitTime ) ;
	local maxDivers = 10 ;
	
	local aliveCreatedEntities = {} ;

	while ( true ) do
		timeToNextEmit = timeToNextEmit - getDeltaT( ) ;
		
		if ( timeToNextEmit < 0.0 ) then
			local emitIndex = math.floor(randomInt( 1, tablegetn(ships) + 1 )) ;			
			local shipName = ships[ emitIndex ] ;

			removeDeadFromList( aliveCreatedEntities ) ;
			local numAlive = tablegetn( aliveCreatedEntities ) ;

			if ( numAlive < maxDivers and entityIsAlive( shipName ) ) then
				--local entityPosX, entityPosY = getEntityPos( shipName ) ;
				local entityPosX, entityPosY = getShotPos( shipName, -0.3, 0.1 ) ;
				local velX = 0.0 ;
				local velY = 0.0 ;
				local angularVel = randomDouble( -40.0, 40.0 ) ;
				local priority = 0 ; -- below
				local scriptName = createEntity( emitObject, entityPosX, entityPosY, 0.0, velX, velY, area, angularVel, priority ) ;
				
				table.insert( aliveCreatedEntities, scriptName ) ;

				--entityAddRandomAreaController( scriptName, "divearea1", 2.5, 8.0 ) ;
			end

			timeToNextEmit = randomDouble( minEmitTime, maxEmitTime ) ;
		end
		
		coroutine.yield( ) ;		
	end
	
	removeScriptedEntityEmitting( emitObject ) ;
end

-- pacific scripts
function seaotter1_2_levelscript( )
	DingoAlert( "Running seaotter1_2 Script" ) ;

	local startingLevel = getCurrentLevelName( ) ;

	ships = { 'otter1', 'otter2', 'otter3' } ;	
	
	local emitObject = "seaurchin_seaotter1_1" ;
	local area = 800.0 ;
	addScriptedEntityEmitting( emitObject, area ) ;

	local minEmitTime = 0.5 ;
	local maxEmitTime = 1.5 ;
	local timeToNextEmit = randomDouble( minEmitTime, maxEmitTime ) ;

	while ( true ) do
		timeToNextEmit = timeToNextEmit - getDeltaT( ) ;
		
		if ( timeToNextEmit < 0.0 ) then
			local emitIndex = math.floor(randomInt( 1, tablegetn(ships) + 1 )) ;			
			local shipName = ships[ emitIndex ] ;

			if ( entityIsAlive( shipName ) ) then
				local entityPosX, entityPosY = getShotPos( shipName, 0.2, -0.1 ) ;
				local velX = 0.0 ;
				local velY = 0.0 ;
				local angularVel = randomDouble( -40.0, 40.0 ) ;
				local priority = 0 ; -- below
				createEntity( emitObject, entityPosX, entityPosY, 0.0, velX, velY, area, angularVel, priority ) ;
			end

			timeToNextEmit = randomDouble( minEmitTime, maxEmitTime ) ;
		end
		
		coroutine.yield( ) ;		
	end
	
	removeScriptedEntityEmitting( emitObject ) ;
end

function oil5_1_levelscript( )
	DingoAlert( "Running oil5_1 Script" ) ;

	local startingLevel = getCurrentLevelName( ) ;

	ships = { 'boat1', 'boat2', 'boat3' } ;	
	
	local emitObject = "oilbubble_oil5_1" ;
	local area = 1000.0 ; --128.0 ;
	addScriptedEntityEmitting( emitObject, area ) ;

	local minEmitTime = 2.0 ;
	local maxEmitTime = 4.0 ;
	local timeToNextEmit = randomDouble( minEmitTime, maxEmitTime ) ;
	local maxDivers = 10 ;
	
	local aliveCreatedEntities = {} ;

	while ( true ) do
		timeToNextEmit = timeToNextEmit - getDeltaT( ) ;
		
		if ( timeToNextEmit < 0.0 ) then
			local emitIndex = math.floor(randomInt( 1, tablegetn(ships) + 1 )) ;			
			local shipName = ships[ emitIndex ] ;

			removeDeadFromList( aliveCreatedEntities ) ;
			local numAlive = tablegetn( aliveCreatedEntities ) ;

			if ( numAlive < maxDivers and entityIsAlive( shipName ) ) then
				--local entityPosX, entityPosY = getEntityPos( shipName ) ;
				local entityPosX, entityPosY = getShotPos( shipName, -0.4, 0.2 ) ;
				local velX = 0.0 ;
				local velY = 0.0 ;
				local angularVel = randomDouble( -40.0, 40.0 ) ;
				local priority = 0 ; -- below
				local scriptName = createEntity( emitObject, entityPosX, entityPosY, 0.0, velX, velY, area, angularVel, priority ) ;
				
				table.insert( aliveCreatedEntities, scriptName ) ;

				--entityAddRandomAreaController( scriptName, "divearea1", 2.5, 8.0 ) ;
			end

			timeToNextEmit = randomDouble( minEmitTime, maxEmitTime ) ;
		end
		
		coroutine.yield( ) ;		
	end
	
	removeScriptedEntityEmitting( emitObject ) ;
end

function clumsycat1_1_levelscript( )
	DingoAlert( "Running clumsycat1_1 Script" ) ;

	local emitObject = "nothing" ;
	local area = 100.0 ; --128.0 ;
	addScriptedEntityEmitting( emitObject, area ) ;

	trackLastEntityPositions( ) ;

	entityList = { 'chair1', 'chair2', 'chair3', 'chair4', 'chair5', 'chair6', 'chair7', 'chair8', 'chair9', 'chair10', 'chair11', 'chair12', 'chair13', 'chair14', 'chair15', 'chair16', 'chair17', 'chair18', 'chair19', 'chair20', 'chair21', 'chair22', 'table1', 'table2', 'table3', 'table4', 'table5', 'table6', 'table7', 'table8', 'table9', 'table10', 'table11', 'counter1', 'counter2', 'counter3', 'counter4', 'counter5', 'counter6', 'counter7', 'counter8' } ;	

	while ( tablegetn( entityList ) >= 0 ) do
		local foundAlive = true ;
		while ( foundAlive ) do
			foundAlive = false ;
			for index, entityName in ipairs(entityList) do
				if ( entityIsAlive( entityName ) == false ) then					
					table.remove( entityList, index ) ;
					foundAlive = true ;
					
					local entityPosX, entityPosY = getEntityPos( entityName ) ;
					
					if ( string.find( entityName, "chair" ) ~= nil ) then
						addPoints( 100, entityPosX, entityPosY ) ;
						playSound( "positivechime" ) ;
					elseif ( string.find( entityName, "table" ) ~= nil ) then
						addPoints( 200, entityPosX, entityPosY ) ;
						playSound( "positivechime" ) ;
					elseif ( string.find( entityName, "counter" ) ~= nil ) then
						addPoints( 400, entityPosX, entityPosY ) ;
						playSound( "positivechime" ) ;
					end
					
					addSpecialEntityEaten( ) ;
					
					break ;
				end
			end
		end
		
		wait( 0.5 ) ;
	end
	
	-- destroyed everything
	
	
	removeScriptedEntityEmitting( emitObject ) ;
end

function clumsycat2_1_levelscript( )
	DingoAlert( "Running clumsycat2_1 Script" ) ;

	trackLastEntityPositions( ) ;

	local emitObject = "nothing" ;
	local area = 100.0 ; --128.0 ;
	addScriptedEntityEmitting( emitObject, area ) ;

	counterName = "counter1" ;
	
	addPointsNoPopUp( 4000 ) ; -- so that points can't be taken away

	local finishedPushing = false ;
	local failed = false ;
	while ( not finishedPushing and not failed) do
		if ( entityIsAlive( counterName ) ) then
			local entityPosX, entityPosY = getEntityPos( counterName ) ;
			if ( entityPosX > 2050.0 ) then
				addPoints( 4000, entityPosX, entityPosY ) ;
				playSound( "positivechime" ) ;
				addSpecialEntityEaten( ) ;
				finishedPushing = true ;
			end	
		else
			failed = true ;
		end
		
		wait( 0.5 ) ;
	end

	if ( failed ) then	
		removeScriptedEntityEmitting( emitObject ) ;
		setFailedByScript( "clumsycat2fail" ) ;
	else
		while ( true ) do
			wait( 0.5 ) ;
		end
	end
end

function droppingbread1_1_levelscript( )
	DingoAlert( "Running dropping1_1 Script" ) ;

	local startingLevel = getCurrentLevelName( ) ;

	ships = { 'cyclist1', 'cyclist2', 'cyclist3', 'cyclist4', 'cyclist5', 'cyclist6', 'parisian1', 'parisian2', 'parisian3', 'parisian4' } ;	
	
	local emitObject = "croissant_base" ;
	local area = 500.0 ; --128.0 ;
	addScriptedEntityEmitting( emitObject, area ) ;

	local minEmitTime = 0.2 ;
	local maxEmitTime = 1.0 ;
	local timeToNextEmit = randomDouble( minEmitTime, maxEmitTime ) ;
	local maxDivers = 14 ;
	
	local aliveCreatedEntities = {} ;

	while ( true ) do
		timeToNextEmit = timeToNextEmit - getDeltaT( ) ;
		
		if ( timeToNextEmit < 0.0 ) then
			local emitIndex = math.floor(randomInt( 1, tablegetn(ships) + 1 )) ;			
			local shipName = ships[ emitIndex ] ;

			removeDeadFromList( aliveCreatedEntities ) ;
			local numAlive = tablegetn( aliveCreatedEntities ) ;

			if ( numAlive < maxDivers and entityIsAlive( shipName ) ) then
				--local entityPosX, entityPosY = getEntityPos( shipName ) ;
				local entityPosX, entityPosY = getShotPos( shipName, -0.1, 0.0 ) ;
				local velX = 0.0 ;
				local velY = 0.0 ;
				local angularVel = randomDouble( -40.0, 40.0 ) ;
				local priority = 0 ; -- below

				local useName = emitObject ;
				local useArea = area ;				
				if ( randomDouble( 0.0, 1.0 ) < 0.5 ) then
					useName = "baguette_base" ;
					useArea = 1000.0 ;
				end

				local scriptName = createEntity( useName, entityPosX, entityPosY, randomDouble(0.0, 360.0 ), velX, velY, useArea, angularVel, priority ) ;
				
				table.insert( aliveCreatedEntities, scriptName ) ;

				--entityAddRandomAreaController( scriptName, "divearea1", 2.5, 8.0 ) ;
			end

			timeToNextEmit = randomDouble( minEmitTime, maxEmitTime ) ;
		end
		
		coroutine.yield( ) ;		
	end
	
	removeScriptedEntityEmitting( emitObject ) ;
end

-- Rat Levels
function beetleGoalieScript( )
	local goalieSpeed = 200.0 ; --400.0 ;
	
	if ( isBoosterSlowEnemiesActive( ) ) then
		goalieSpeed = goalieSpeed / 5.0 ;
	end
	
	local goalieName = "beetle1" ;

	local goalieDefaultX = 900.0 ;
	local goalieDefaultY = 0.0 ;

	while ( true ) do
		if ( entityIsAlive( goalieName ) ) then
			local goaliePosX, goaliePosY = getEntityPos( goalieName ) ;

			local destX = goalieDefaultX ;
			local destY = goalieDefaultY ;

			if ( entityIsAlive( dungBallName ) ) then
				local ballPosX, ballPosY = getEntityPos( dungBallName ) ;
				if ( ballPosX > 350.0 and ballPosX < 1000.0 and ballPosY > -560.0 and ballPosY < 530.0 ) then
					-- ball inside tracking zone
					destX = ballPosX ;
					destY = ballPosY ;
				end
			end

			local dirX = destX - goaliePosX ;
			local dirY = destY - goaliePosY ;

			local desiredAngleDegrees = 0.0 ;
			
			local dist = math.sqrt( dirX * dirX + dirY * dirY ) ;
			if ( dist < 10.0 ) then
				setEntityVel( goalieName, 0.0, 0.0 ) ;
				desiredAngleDegrees = 180.0 ;
			else
				local normalizedX = dirX / dist ;
				local normalizedY = dirY / dist ;
				
				local velX = normalizedX * goalieSpeed ;
				local velY = normalizedY * goalieSpeed ;
				
				setEntityVel( goalieName, velX, velY ) ;
				
				local angleToDest = mathatan2( -dirY, dirX ) ;
				desiredAngleDegrees = angleToDest * 57.2958  ;
			end

			local transSpeed = 0.05 ;	
			local currentAngle = getEntityAngle( goalieName ) ;
			local useDesiredAngleDegrees = currentAngle + angleDifference( desiredAngleDegrees, currentAngle ) * transSpeed ;
			setEntityAngle( goalieName, useDesiredAngleDegrees ) ;
		end
		
		coroutine.yield( ) ;
	end
end

function beetleDefenderScript( goalieName, defaultPosX, defaultPosY )
	local goalieSpeed = 200.0 ; --400.0 ;
	
	if ( isBoosterSlowEnemiesActive( ) ) then
		goalieSpeed = goalieSpeed / 5.0 ;
	end
	
	local goalieDefaultX = defaultPosX ;
	local goalieDefaultY = defaultPosY ;
	
	local rangeWidth = 400.0 ;
	local rangeX1 = goalieDefaultX - rangeWidth * 0.5 ;
	local rangeX2 = goalieDefaultX + rangeWidth * 0.5 ;

	local rangeHeight = 800.0 ;
	local rangeY1 = goalieDefaultY - rangeHeight * 0.5 ;
	local rangeY2 = goalieDefaultY + rangeHeight * 0.5 ;

	while ( true ) do
		if ( entityIsAlive( goalieName ) ) then
			local goaliePosX, goaliePosY = getEntityPos( goalieName ) ;

			local destX = goalieDefaultX ;
			local destY = goalieDefaultY ;

			if ( entityIsAlive( dungBallName ) ) then
				local ballPosX, ballPosY = getEntityPos( dungBallName ) ;
				if ( ballPosX > rangeX1 and ballPosX < rangeX2 and ballPosY > rangeY1 and ballPosY < rangeY2 ) then
					-- ball inside tracking zone
					destX = ballPosX ;
					destY = ballPosY ;
				end
			end

			local dirX = destX - goaliePosX ;
			local dirY = destY - goaliePosY ;

			local desiredAngleDegrees = 0.0 ;
			
			local dist = math.sqrt( dirX * dirX + dirY * dirY ) ;
			if ( dist < 10.0 ) then
				setEntityVel( goalieName, 0.0, 0.0 ) ;
				desiredAngleDegrees = 180.0 ;
			else
				local normalizedX = dirX / dist ;
				local normalizedY = dirY / dist ;
				
				local velX = normalizedX * goalieSpeed ;
				local velY = normalizedY * goalieSpeed ;
				
				setEntityVel( goalieName, velX, velY ) ;
				
				local angleToDest = mathatan2( -dirY, dirX ) ;
				desiredAngleDegrees = angleToDest * 57.2958  ;
			end

			local transSpeed = 0.05 ;	
			local currentAngle = getEntityAngle( goalieName ) ;
			local useDesiredAngleDegrees = currentAngle + angleDifference( desiredAngleDegrees, currentAngle ) * transSpeed ;
			setEntityAngle( goalieName, useDesiredAngleDegrees ) ;
		end
		
		coroutine.yield( ) ;
	end
end

function dungsoccer1_1_levelscript( )
	local scriptName = "dungsoccer1_1" ;
	printStartScript( scriptName ) ;

	trackLastEntityPositions( ) ;

	local emitName = "dungball_soccer1_1" ;
	addScriptedEntityEmitting( emitName, 1.0 ) ;

	local minTimeShot = 0.0 ;
	local maxTimeShot = 0.7 ;

	dungBallName = "" ;

	activateScriptMessage( "beetleGoalieScript" ) ;

	while ( true ) do
		local soccerBallArea = 4000.0 ; --2600.0 ;	

		local ballLifeTime = 0.0 ;
		
		local posX = -180.0 ;
		local posY = -870.0 ;

		local speed = 10.0 ; --2.0 ;	
		local launchAngleDegrees = 270.0 ;
		local launchAngleRadians = launchAngleDegrees * ( 3.14159/180.0 ) ;
		local velX = speed * math.cos( launchAngleRadians ) ;
		local velY = - speed * math.sin( launchAngleRadians ) ;	
		
		local angularVelocity = randomDouble( -300.0, 300.0 ) ;
		local priority = math.floor(randomInt( 100, 130 )) ;
		local startAngle = randomDouble( -100.0, 100.0 ) ;
		dungBallName = createEntity( emitName, posX, posY, startAngle, velX, velY, soccerBallArea, angularVelocity, priority ) ;
		setEntityPoints( dungBallName, true, 1 ) ;
		
		local ballDone = false ;
		while ( entityIsAlive( dungBallName ) and ( not ballDone ) ) do
			local ballPosX, ballPosY = getEntityPos( dungBallName ) ;
			
			if ( ballPosY > -510.0 and ballPosY < 460.0 and ballPosX > 1000.0 ) then
				-- scored
				if ( getScore( ) < 10000 ) then -- don't let player keep scoring
					addPoints( 250, 900.0, ballPosY ) ;
					playSound( "positivechime" ) ;
					addSpecialEntityEaten( ) ;
				end
				ballDone = true ;
			elseif ( ballPosX < -990.0 or ballPosX > 1005.0 or ballPosY < -790.0 or ballPosY > 790.0 ) then
				-- ball almost outside level
				local checkOutOfBoundsAfter = 2.0 ;
				if ( getEntityLivedFor( dungBallName ) > checkOutOfBoundsAfter ) then
					destX = ballPosX ;
					destY = ballPosY ;
					ballDone = true ;
				end
			end

			coroutine.yield( ) ;
		end
		
		wait( 0.5 ) ;
	end
	
	removeScriptedEntityEmitting( emitName ) ;
	
	printEndScript( scriptName ) ;
end

function dungsoccer2_1_levelscript( )
	local scriptName = "dungsoccer2_1" ;
	printStartScript( scriptName ) ;

	trackLastEntityPositions( ) ;

	local emitName = "dungball_soccer2_1" ;
	addScriptedEntityEmitting( emitName, 1.0 ) ;

	local minTimeShot = 0.0 ;
	local maxTimeShot = 0.7 ;

	dungBallName = "" ;

	activateScriptMessage( "beetleGoalieScript" ) ;
	activateScriptMessage( "beetleDefenderScript", "beetle2", 132.0, 415.0 ) ;
	activateScriptMessage( "beetleDefenderScript", "beetle3", 113.0, -358.0 ) ;
	activateScriptMessage( "beetleDefenderScript", "beetle4", -492.0, 22.0 ) ;
	activateScriptMessage( "beetleDefenderScript", "beetle5", -1117.0, 434.0 ) ;
	activateScriptMessage( "beetleDefenderScript", "beetle6", -1131.0, -355.0 ) ;

	while ( true ) do
		local soccerBallArea = 4000.0 ; --2600.0 ;	

		local ballLifeTime = 0.0 ;
		
		local posX = -1900.0 ;
		local posY = 0.0 ;

		local speed = 5.0 ; --2.0 ;	
		local launchAngleDegrees = 0.0 ;
		local launchAngleRadians = launchAngleDegrees * ( 3.14159/180.0 ) ;
		local velX = speed * math.cos( launchAngleRadians ) ;
		local velY = - speed * math.sin( launchAngleRadians ) ;	
		
		local angularVelocity = randomDouble( -300.0, 300.0 ) ;
		local priority = math.floor(randomInt( 100, 130 )) ;
		local startAngle = randomDouble( -100.0, 100.0 ) ;
		dungBallName = createEntity( emitName, posX, posY, startAngle, velX, velY, soccerBallArea, angularVelocity, priority ) ;
		setEntityPoints( dungBallName, true, 1 ) ;
		
		local ballDone = false ;
		while ( entityIsAlive( dungBallName ) and ( not ballDone ) ) do
			local ballPosX, ballPosY = getEntityPos( dungBallName ) ;
			
			if ( ballPosY > -510.0 and ballPosY < 460.0 and ballPosX > 1000.0 ) then
				-- scored
				if ( getScore( ) < 10000 ) then -- don't let player keep scoring
					addPoints( 250, 900.0, ballPosY ) ;
					playSound( "positivechime" ) ;
					addSpecialEntityEaten( ) ;
				end
				ballDone = true ;
			elseif ( ballPosX < -1790.0 or ballPosX > 1005.0 or ballPosY < -790.0 or ballPosY > 790.0 ) then
				-- ball almost outside level
				local checkOutOfBoundsAfter = 2.0 ;
				if ( getEntityLivedFor( dungBallName ) > checkOutOfBoundsAfter ) then
					destX = ballPosX ;
					destY = ballPosY ;
					ballDone = true ;
				end
			end

			coroutine.yield( ) ;
		end
		
		wait( 0.5 ) ;
	end
	
	removeScriptedEntityEmitting( emitName ) ;
	
	printEndScript( scriptName ) ;
end

function poachermine1_1_levelscript( )
	local scriptName = "poachermine1_1" ;
	printStartScript( scriptName ) ;

	local emitName = "tempnothingsmall" ;
	addScriptedEntityEmitting( emitName, 1.0 ) ;

	local minTimeShot = 0.0 ;
	local maxTimeShot = 0.7 ;

	while ( true ) do
		wait( 0.5 ) ;
	end
	
	removeScriptedEntityEmitting( emitName ) ;
	
	printEndScript( scriptName ) ;
end

function protectbush1_1_levelscript( )
	local scriptName = "protectbush1_1" ;
	printStartScript( scriptName ) ;

	local bushName = "bush1"

	addPointsNoPopUp( 5000 ) ; -- start with points, so if we eat bush right away we lose them

	local bushPosX, bushPosY = getEntityPos( bushName ) ;

	while ( entityIsAlive( bushName ) ) do
		wait( 0.1 ) ;
	end

	removePoints( 5000, bushPosX, bushPosY ) ;
	playSound( "negativechime" ) ;
	
	printEndScript( scriptName ) ;
end

function brickstack3_1_levelscript( )
	DingoAlert( "Running brickstack3_1 Script" ) ;

	local startingLevel = getCurrentLevelName( ) ;

	local eatBricks = { 'eatbrick1', 'eatbrick2', 'eatbrick3' } ;	
	local pushBricks = { 'pushbrick1', 'pushbrick2', 'pushbrick3' } ;	

	local pushBrick1StartX, pushBrick1StartY = getEntityPos( pushBricks[1] ) ;
	local pushBrick2StartX, pushBrick2StartY = getEntityPos( pushBricks[2] ) ;
	local pushBrick3StartX, pushBrick3StartY = getEntityPos( pushBricks[3] ) ;
	
	local pushBrick1Moved = false ;
	local pushBrick2Moved = false ;
	local pushBrick3Moved = false ;
	
	local movedDist = 8.0 ;
	
	while ( not pushBrick1Moved or not pushBrick2Moved or not pushBrick3Moved ) do
		if ( not pushBrick1Moved ) then
			local posX, posY = getEntityPos( pushBricks[1] ) ;
			local diffX = posX - pushBrick1StartX ;
			local diffY = posY - pushBrick1StartY ;
			local dist = math.sqrt( diffX*diffX + diffY*diffY ) ;
			
			if ( dist > movedDist ) then
				pushBrick1Moved = true ;
				DingoAlert( "pushed brick 1 " .. dist ) ;
			end
		end
		
		if ( not pushBrick2Moved ) then
			local posX, posY = getEntityPos( pushBricks[2] ) ;
			local diffX = posX - pushBrick2StartX ;
			local diffY = posY - pushBrick2StartY ;
			local dist = math.sqrt( diffX*diffX + diffY*diffY ) ;
			
			if ( dist > movedDist ) then
				pushBrick2Moved = true ;
				DingoAlert( "pushed brick 2 " .. dist ) ;
			end
		end
		
		if ( not pushBrick3Moved ) then
			local posX, posY = getEntityPos( pushBricks[3] ) ;
			local diffX = posX - pushBrick3StartX ;
			local diffY = posY - pushBrick3StartY ;
			local dist = math.sqrt( diffX*diffX + diffY*diffY ) ;
			
			if ( dist > movedDist ) then
				pushBrick3Moved = true ;
				DingoAlert( "pushed brick 3 " .. dist ) ;
			end
		end
		
		coroutine.yield( ) ;		
	end

	DingoAlert( "touched all three" ) ;
	
	local timeSincePushed = 0.0 ;
	local maxTimeSincePush = 30.0 ;

	while ( ( timeSincePushed < maxTimeSincePush ) and 
		( entityIsAlive( eatBricks[1] ) or entityIsAlive( eatBricks[2] ) or entityIsAlive( eatBricks[3] ) ) ) do
		timeSincePushed = timeSincePushed + getDeltaT( ) ;
		coroutine.yield( ) ;			
	end
	
	DingoAlert( "all are not alive or time is up" ) ;

	wait( 5.0 ) ; -- all bricks gone or time elapsed since touching push bricks
	setFailedByScript( "brickstack3fail" ) ;
	
	DingoAlert( "Finished brickstack3_1 Script" ) ;
end

function pushgarbage1_1_levelscript( )
	DingoAlert( "Running pushgarbage1_1 Script" ) ;

	local startingLevel = getCurrentLevelName( ) ;

	trackLastEntityPositions( ) ;

	local emitName = "tempnothingsmall" ;
	addScriptedEntityEmitting( emitName, 1.0 ) ;

	local bagList = { 'bag1', 'bag2', 'bag3', 'bag4', 'bag5', 'bag6', 'bag7', 'bag8', 'bag9', 'bag10' } ;	
	local boxList = { 'box1', 'box2', 'box3', 'box4', 'box5', 'box6', 'box7', 'box8', 'box9', 'box10' } ;	

	local garbageLeft = -220.0 ;
	local garbageRight = 80.0 ;
	local garbageTop = 500.0 ;
	local garbageBottom = 685.0 ;
	
	local recyclingLeft = 135.0 ;
	local recyclingRight = 445.0 ;
	local recyclingTop = 500.0 ;
	local recyclingBottom = 685.0 ;
	
	local levelBottom = 1500.0 ;
	
	while ( true ) do
	
		for index, name in ipairs(bagList) do
			local posX, posY = getEntityPos( name ) ;
			if ( posX > garbageLeft and posX < garbageRight and posY > garbageTop and posY < garbageBottom ) then
				addPoints( 500, posX, posY ) ;
				playSound( "positivechime" ) ;
				addSpecialEntityEaten( ) ;
				removeEntity( name ) ;
				table.remove( bagList, index ) ; -- this will cause next element to be skipped right now - but I think it's safe to do
			elseif ( posX > recyclingLeft and posX < recyclingRight and posY > recyclingTop and posY < recyclingBottom ) then
				removePoints( 500, posX, posY ) ;
				playSound( "negativechime" ) ;
				addSpecialEntityEaten( ) ;
				removeEntity( name ) ;
				table.remove( bagList, index ) ; -- this will cause next element to be skipped right now - but I think it's safe to do
			elseif ( posY > levelBottom ) then
				removePoints( 1000, posX, posY ) ; -- missed by a lot
				playSound( "negativechime" ) ;
				addSpecialEntityEaten( ) ;
				removeEntity( name ) ;
				table.remove( bagList, index ) ; -- this will cause next element to be skipped right now - but I think it's safe to do				
			end
		end
		
		for index, name in ipairs(boxList) do
			local posX, posY = getEntityPos( name ) ;
			if ( posX > garbageLeft and posX < garbageRight and posY > garbageTop and posY < garbageBottom ) then
				removePoints( 500, posX, posY ) ;
				playSound( "negativechime" ) ;
				addSpecialEntityEaten( ) ;
				removeEntity( name ) ;
				table.remove( boxList, index ) ; -- this will cause next element to be skipped right now - but I think it's safe to do
			elseif ( posX > recyclingLeft and posX < recyclingRight and posY > recyclingTop and posY < recyclingBottom ) then
				addPoints( 500, posX, posY ) ;
				playSound( "positivechime" ) ;
				addSpecialEntityEaten( ) ;
				removeEntity( name ) ;
				table.remove( boxList, index ) ; -- this will cause next element to be skipped right now - but I think it's safe to do
			elseif ( posY > levelBottom ) then
				removePoints( 1000, posX, posY ) ; -- missed by a lot
				playSound( "negativechime" ) ;
				addSpecialEntityEaten( ) ;
				removeEntity( name ) ;
				table.remove( boxList, index ) ; -- this will cause next element to be skipped right now - but I think it's safe to do				
			end
		end

		wait( 0.5 ) ;
		coroutine.yield( ) ;
	end
	
	removeScriptedEntityEmitting( emitName ) ;
	
	DingoAlert( "Finished pushgarbage1_1 Script" ) ;
end

function windturbines1_1_levelscript( )
	DingoAlert( "Running windturbines1_1 Script" ) ;

	addPointsNoPopUp( 20000 ) ; -- so that points can be taken away

	DingoAlert( "Ending windturbines1_1 Script" ) ;
end

function forcePlayersCloseToStart( properStartX, properStartY, maxStartDist )
	if ( getNumPlayers( ) == 2 ) then
		for var=0,(getNumPlayers( )-1) do
			local playerName = "player_" .. var ;
			local entityPosX, entityPosY = getEntityPos( playerName ) ;
			
			local diffX = properStartX - entityPosX ;
			local diffY = properStartY - entityPosY ;
			
			local dist = math.sqrt( diffX * diffX + diffY * diffY ) ;
			if ( dist > maxStartDist ) then
				DingoAlert( "moving " .. playerName .. " to proper start" ) ;
				setEntityPos( playerName, properStartX, properStartY ) ;
			end
		end
	end
end

function france1_4_levelscript( )
	DingoAlert( "Running france1_4 Script" ) ;
	
	local properStartX = -1433.0 ;
	local properStartY = -75.0 ;
	local maxStartDist = 121.0 ; -- force to start if further than this

	forcePlayersCloseToStart( properStartX, properStartY, maxStartDist ) ;
	
	DingoAlert( "Finished france1_4 Script" ) ;
end

function mars1_24_levelscript( )
	DingoAlert( "Running mars1_24 Script" ) ;
	
	local properStartX = 104.0 ;
	local properStartY = 1.0 ;
	local maxStartDist = 62.0 ; -- force to start if further than this

	forcePlayersCloseToStart( properStartX, properStartY, maxStartDist ) ;

	DingoAlert( "Finished mars1_24 Script" ) ;
end

function elephantmines1_1_levelscript( )
	DingoAlert( "Running elephantmines1_1 Script" ) ;

	while ( true ) do
		if ( getNumSpecialsMoreToBeatLevel( ) > getNumSpecialsInLevel( ) ) then
			setFailedByScript( "elephantmines1immediatefail" ) ;
		end
		
		wait( 1.0 ) ;
	end	

	DingoAlert( "Finished elephantmines1_1 Script" ) ;
end
